'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _spawnCommandWithKill = require('spawn-command-with-kill');

var _spawnCommandWithKill2 = _interopRequireDefault(_spawnCommandWithKill);

var _pinkiePromise = require('pinkie-promise');

var _pinkiePromise2 = _interopRequireDefault(_pinkiePromise);

var _async = require('async');

var _async2 = _interopRequireDefault(_async);

var _safe = require('colors/safe');

var _safe2 = _interopRequireDefault(_safe);

var _lodash = require('lodash');

var _findUp = require('find-up');

var _managePath = require('manage-path');

var _managePath2 = _interopRequireDefault(_managePath);

var _arrify = require('arrify');

var _arrify2 = _interopRequireDefault(_arrify);

var _getScriptToRun = require('./get-script-to-run');

var _getScriptToRun2 = _interopRequireDefault(_getScriptToRun);

var _getScriptsFromConfig = require('./get-scripts-from-config');

var _getScriptsFromConfig2 = _interopRequireDefault(_getScriptsFromConfig);

var _getLogger = require('./get-logger');

var _getLogger2 = _interopRequireDefault(_getLogger);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var noop = function noop() {}; // eslint-disable-line func-style, no-empty-function

exports.default = runPackageScripts;


function runPackageScripts(_ref) {
  var scriptConfig = _ref.scriptConfig;
  var scripts = _ref.scripts;
  var args = _ref.args;
  var _ref$options = _ref.options;
  var options = _ref$options === undefined ? {} : _ref$options;
  var callback = arguments.length <= 1 || arguments[1] === undefined ? noop : arguments[1];

  if (scripts.length === 0) {
    scripts = ['default'];
  }
  var scriptNames = (0, _arrify2.default)(scripts);
  if (options.parallel) {
    runParallel();
  } else {
    runSeries();
  }

  function runSeries() {
    _async2.default.mapSeries(scriptNames, function (scriptName, cb) {
      runPackageScript({ scriptConfig: scriptConfig, options: options, scriptName: scriptName, args: args }).then(function (code) {
        return cb(null, code);
      }, cb);
    }, onFinished);
  }

  function runParallel() {
    var results = scriptNames.map(function (script) {
      return { script: script, code: undefined };
    });
    var aborted = false;
    var errorCode = null;

    var promises = scriptNames.map(function (scriptName) {
      return runPackageScript({ scriptConfig: scriptConfig, options: options, scriptName: scriptName, args: args });
    });

    var allPromise = _pinkiePromise2.default.all(function (promise, index) {
      return promise.then(function (code) {
        if (aborted) {
          return;
        }

        results[index].code = code;

        if (errorCode === null && code !== 0) {
          errorCode = errorCode || code;
          abortAll();
        }
      });
    });

    allPromise.catch(function () {
      if (!aborted) {
        abortAll();
      }
    });

    allPromise.then(function () {
      return onFinished(null, results);
    }, onFinished);

    function abortAll() {
      aborted = true;
      promises.forEach(function (p) {
        return p.abort();
      });
    }
  }

  function onFinished(err, results) {
    if (err) {
      callback({ error: err });
    } else {
      (function () {
        var NON_ERROR = 0;
        var result = (0, _lodash.find)(results, function (r) {
          return r !== NON_ERROR;
        });
        callback({ code: result });
      })();
    }
  }
}

function runPackageScript(_ref2) {
  var scriptConfig = _ref2.scriptConfig;
  var options = _ref2.options;
  var scriptName = _ref2.scriptName;
  var args = _ref2.args;

  var scripts = (0, _getScriptsFromConfig2.default)(scriptConfig, scriptName);
  var script = (0, _getScriptToRun2.default)(scripts, scriptName);
  if (!(0, _lodash.isString)(script)) {
    return _pinkiePromise2.default.reject({
      message: _safe2.default.red('Scripts must resolve to strings. There is no script that can be resolved from "' + scriptName + '"'),
      ref: 'missing-script'
    });
  }
  var command = [script, args].join(' ').trim();
  var log = (0, _getLogger2.default)(getLogLevel(options));
  log.info(_safe2.default.gray('p-s executing: ') + _safe2.default.green(command));
  var child = void 0;
  var promise = new _pinkiePromise2.default(function (resolve, reject) {
    child = (0, _spawnCommandWithKill2.default)(command, { stdio: 'inherit', env: getEnv() });

    child.on('error', function (err) {
      child = null;
      reject(err);
    });

    child.on('close', function (code) {
      child = null;
      resolve(code);
    });
  });

  promise.abort = function abort() {
    if (child !== null) {
      child.kill();
      child = null;
    }
  };

  return promise;
}

function getLogLevel(_ref3) {
  var silent = _ref3.silent;
  var logLevel = _ref3.logLevel;

  if (logLevel) {
    return logLevel;
  } else if (silent) {
    return 'disable';
  } else {
    return undefined;
  }
}

function getEnv() {
  var env = (0, _lodash.clone)(process.env);
  var alterPath = (0, _managePath2.default)(env);
  var npmBin = (0, _findUp.sync)('node_modules/.bin');
  if (npmBin) {
    alterPath.unshift(npmBin);
  }
  return env;
}